require(MASS)
require(parallel)
SIM_LENGTH <- 10000
NUM_CORES <- detectCores()

create_cov <- function(n,sigma1,sigma2,theta,lambda) {
	tmp1 <- matrix(rep(theta,n*n),nrow=n)*sigma1*sigma1
	diag(tmp1) <- sigma1*sigma1
	retval <- diag(2*n)*sigma2*sigma2
	retval[1:n,1:n] <- tmp1
	diag(retval[(1):(n),(n+1):(2*n)])<-lambda*sigma1*sigma2
	diag(retval[(n+1):(2*n),(1):(n)])<-lambda*sigma1*sigma2
	return(retval)
}

getval <- function(nA=50,mua=1,sigma1A=1,sigma2A=1,thetaA=0.2,lambdaA=0.1,nB=50,mub=1,sigma1B=1,sigma2B=1,thetaB=0.2,lambdaB=0.1,R=0.1) {
	muA <- c(rep(mua,nA),rep(0,nA))
	muB <- c(rep(mub,nB),rep(0,nB))
	covA <- create_cov(nA,sigma1A,sigma2A,thetaA,lambdaA)
	covB <- create_cov(nB,sigma1B,sigma2B,thetaB,lambdaB)

	Avals <- mvrnorm(SIM_LENGTH,muA,covA)

	Bvals <- mvrnorm(SIM_LENGTH,muB,covB)


	outcomes <- cbind(Avals[,1:nA],Bvals[,1:nB])

	numselected <- (nA+nB)*R
	selected <- t(apply(outcomes,1,rank))>(nA+nB-numselected)
	tmp1 <- matrix(outcomes[selected],byrow=TRUE,ncol=numselected)
	outcomes <- cbind(Avals[,(nA+1):(2*nA)],Bvals[,(nB+1):(2*nB)])
	finalval <- tmp1 + matrix(outcomes[selected],byrow=TRUE,ncol=numselected)

	mean(apply(finalval,1,sum))
}



draw_loess <- function(deln1n2,val,mcol="red",linetype=2) {

	tmp<-loess(val~deln1n2)
	f<-predict(tmp,deln1n2)

	lines(deln1n2,f,type="l",lwd=2,col=mcol,lty=linetype);
}



#########heterogeneity in means - optimal skew

deln1n2 <- 2*c(2:98)-100

finalval0 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA)},mc.cores=NUM_CORES))
finalval1 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA,mua=1.1)},mc.cores=NUM_CORES))


pdf("../Figures/val0.pdf",family="Times")
plot(deln1n2,finalval1,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=1,cex=0.75)
draw_loess(deln1n2,finalval1,mcol="red",linetype=6)
points(deln1n2,finalval0,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(deln1n2,finalval0,mcol="green",linetype=1)
legend("topleft",expression(paste("",mu[A]," =1   , ",mu[B], "=1"),paste("",mu[A]," =1.1, ",mu[B], "=1")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()


#########heterogeneity in variances - optimal skew

deln1n2 <- 2*c(2:98)-100
finalval0 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA,sigma1A=1)},mc.cores=NUM_CORES))
finalval1 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA,sigma1A=1.1)},mc.cores=NUM_CORES))


pdf("../Figures/valsig0.pdf",family="Times")
plot(deln1n2,finalval1,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=1,cex=0.75)
draw_loess(deln1n2,finalval1,mcol="red",linetype=6)
points(deln1n2,finalval0,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(deln1n2,finalval0,mcol="green",linetype=1)
legend("topleft",expression(paste("",sigma[1][A]," =1   , ",sigma[1][B], "=1"),paste("",sigma[1][A]," =1.1, ",sigma[1][B], "=1")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()

#########heterogeneity in correlation theta

deln1n2 <- 2*c(2:98)-100

finalval0 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA,thetaA=0.2)},mc.cores=NUM_CORES))
finalval1 <- unlist(mclapply(c(2:98),function(nA) {print(nA);flush.console();getval(nA=nA,nB=100-nA,thetaA=0.6)},mc.cores=NUM_CORES))


pdf("../Figures/valtheta0.pdf",family="Times")
plot(deln1n2,finalval1,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=1,cex=0.75,ylim=c(24,28.5))
draw_loess(deln1n2,finalval1,mcol="red",linetype=6)
points(deln1n2,finalval0,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(deln1n2,finalval0,mcol="green",linetype=1)
legend("topleft",expression(paste("",theta[A]," =0.2, ",theta[B], "=0.2"),paste("",theta[A]," =0.6, ",theta[B], "=0.2")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()

#########heterogeneity in correlation lambda

deln1n2 <- 2*c(2:98)-100
finalval0 <- unlist(mclapply(c(2:98),function(nA){print(nA);flush.console();getval(nA=nA,nB=100-nA,lambdaA=0.1)},mc.cores=NUM_CORES))
finalval1 <- unlist(mclapply(c(2:98),function(nA){print(nA);flush.console();getval(nA=nA,nB=100-nA,lambdaA=0.2)},mc.cores=NUM_CORES))


pdf("../Figures/vallambda0.pdf",family="Times")
plot(deln1n2,finalval1,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=1,cex=0.75,ylim=c(27,30))
draw_loess(deln1n2,finalval1,mcol="red",linetype=6)
points(deln1n2,finalval0,xlab=expression(n[A]-n[B]),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(deln1n2,finalval0,mcol="green",linetype=1)
legend("topleft",expression(paste("",lambda[A]," =0.1, ",lambda[B], "=0.1"),paste("",lambda[A]," =0.2, ",lambda[B], "=0.1")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()

#########different lambda-balanced vs. skewed.

##lambda cannot be larger than sigma1A*(1-thetaA)/sigma2A
lambda_vals <- 1*(1-0.2)/1 * c(0:10)/10
balancedV <- unlist(mclapply(lambda_vals,function(lambda) {print(lambda);flush.console();getval(mua=1.1,mub=1,lambdaA=lambda,lambdaB=lambda)},mc.cores=NUM_CORES))
skewV <- unlist(mclapply(lambda_vals,function(lambda) {print(lambda);flush.console();getval(nA=98,nB=2,mua=1.1,mub=1,lambdaA=lambda,lambdaB=lambda)},mc.cores=NUM_CORES))


pdf("../Figures/bestshape-lambda.pdf",family="Times")
plot(lambda_vals,balancedV,xlab=expression(lambda),ylab="Quality of top 10% ideas",pch=1,cex=0.75)
draw_loess(lambda_vals,balancedV,mcol="green",linetype=1)
points(lambda_vals,skewV,xlab=expression(lambda),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(lambda_vals,skewV,mcol="red",linetype=6)
legend("topleft",expression(paste("",n[A]," =50 , ",n[B], "=50"),paste("",n[A]," =100, ",n[B], "=0")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()



#########different thetas-balanced vs. skewed.

# maximum value of theta is (1-lambdaA*sigma2A/sigma1A)
theta_vals <- (1-0.1*1/1)*c(0:10)/10

balancedV <- unlist(mclapply(theta_vals,function(theta){print(theta);flush.console();getval(nA=50,nB=50,mua=1.1,mub=1,thetaA=theta,thetaB=theta)},mc.cores=NUM_CORES))
skewV <- unlist(mclapply(theta_vals,function(theta){print(theta);flush.console();getval(nA=98,nB=2,mua=1.1,mub=1,thetaA=theta,thetaB=theta)},mc.cores=NUM_CORES))



pdf("../Figures/bestshape-theta.pdf",family="Times")
plot(theta_vals,skewV,xlab=expression(theta),ylab="Quality of top 10% ideas",pch=6,cex=0.5)
draw_loess(theta_vals,skewV,mcol="red",linetype=6)
points(theta_vals,balancedV,xlab=expression(theta),ylab="Quality of top 10% ideas",pch=1,cex=0.75)
draw_loess(theta_vals,balancedV,mcol="green",linetype=1)
legend("topright",expression(paste("",n[A]," =50 , ",n[B], "=50"),paste("",n[A]," =100, ",n[B], "=0")),cex=0.8,col=c("green","red"),lwd=2,lty=c(1,6))
dev.off()


