require(MASS)
require(clusterGeneration)
require(moments)
require(parallel)

numrepl <- 10000
NUM_CORES <- detectCores()

random_mat <- function(n) {
	#tmp <- rcorrmatrix(n,alphad=1)
	tmp <- genPositiveDefMat(n,covMethod=c("unifcorrmat"),rangeVar=c(1,200))$Sigma
	tmp <- tmp - min(tmp)
	cov2cor(tmp)
}


get_val <- function(covmat) {
	
	n <- dim(covmat)[1]
	
	vals <- mvrnorm(numrepl,rep(0,n),covmat)
	tmp<-t(apply(vals,1,rank)>n*(1-R))
	mean(apply(vals*tmp,1,sum))
}

get_weights <- function(x) {
	x <- (x - mean(x))/sd(x)
	tmp <- matrix(rep(x,length(x)),nrow=length(x))
	tmp <- tmp-t(tmp)
	apply(tmp*tmp,2,sum)
}


draw_loess <- function(deln1n2,val,mcol="red") {

	tmp<-loess(val~deln1n2)
	f<-predict(tmp,deln1n2)

	lines(deln1n2,f,type="l",lwd=2,col=mcol);
}

nDISP <- function(cormat) {
	
	tmp <- cormat[upper.tri(cormat)]
	tmp <- rep(tmp,2)
	tmp <- 1-tmp
	sum(tmp)

}

nUNIFORM <- function(cormat) {
	tmp <- cormat[upper.tri(cormat)]
	tmp <- rep(tmp,2)
	tmp <- 1-tmp
	
	tmp <- tmp - mean(tmp)
	
	tmp <- tmp*tmp
	sum(tmp)
}

R <- 0.1
n <- 40

valfunc <- function(x) {
	print(x)
	flush.console()
	mycor <- random_mat(n)
	cor_vec <- mycor[upper.tri(mycor)]
	return(c(max(eigen(mycor)$values),log(det(mycor)),nDISP(mycor),nUNIFORM(mycor),skewness(cor_vec),kurtosis(cor_vec),get_val(mycor),max(cor_vec),median(cor_vec)))
}

ptm <- proc.time()
tmp <- mclapply(c(1:5000),valfunc,mc.cores=NUM_CORES)
tmp <- data.frame(matrix(unlist(tmp),nrow=length(tmp),byrow=T))
proc.time()-ptm
colnames(tmp) <- c("maxeigen","mdet","DISP","UNIFORM","skew","kurtosis","val","maxcor","medcor")

gmain <- lm(val~DISP+UNIFORM,tmp)
summary(gmain)
